from cgi import print_arguments
from iexcode.instruments.VLS_PGM import mono_energy_get

from epics import caget, caput

from iexcode.instruments.utilities import print_warning_message
from iexcode.instruments.FMB_mirrors import *

##############################################################################################################
################################             M3R             ##############################
##############################################################################################################
M3R_RY_POS_SP_initial = -16.52
M3R_camera = '29id_ps6:'
M3R_cam1  = M3R_camera+"cam1:"
M3R_stats = M3R_camera+"Stats1:"
M3R_pitch = "29id_m3r:RY_MON"
M3R_align_pv = "29idKappa:align_m3r:"

##############################################################################################################
################################             M3R  default positions           ##############################
##############################################################################################################
def m3r_position_dictionary():
    """
    dictionary for the default positions for a given branch
    """
    positions={
        "c":[10,0,0,0,0,0],
        "d":[-2.5,0,0,-13.955,-16.322,-8.5],
        "e":[-2.000,0,0,-13.960,-16.614,-7.500] 
    }


    # d => optimized for MEG @ 500 eV on 2/29/def start
    # e => 2018_3-- JM changed for RSoXS alignment max diode current
    return positions

def m3r_get(verbose=True):    
    position = FMB_mirror_get(3,verbose)
    return position
    
def m3r_RY_pos_sp():
    return M3R_RY_POS_SP_initial

def m3r_RY_pos_sp_get():
    RY_SP = caget('29id_m3r:RY_POS_SP')
    return RY_SP

def m3r_tweak (axis,val):
    """
        \"TX\" = lateral                 \"RX\" = Yaw
        \"TY\" = vertical                \"RY\" = Pitch
        \"TZ\" = longitudinal            \"RZ\" = Roll

    Previously: Tweak_M3R
    """
    mirror_num = 3
    FMB_mirror_tweak(mirror_num,axis,val,verbose=True)

def m3r_tweak_pitch(sign,val=0.005):
    """
    Previously: Tweak_M3R_Pitch
    """
    axis="RY"
    if sign == "+":
        m3r_tweak(axis,val)
    elif sign == "-":
        m3r_tweak(axis,-val)


    
def m3r_table(branch):   
    """
    Defines the positions used by switch_branch() form M3R
    WARNING: branch_pv uses: D => (Tx <= 0) and (Ry < 0) - Make sure this remains true or change it
    """
    positions = m3r_position_dictionary()

    try:
        position = positions[branch]
    except KeyError:
        print_warning_message("not a valid branch")
        print(list(positions.keys()))
        position = FMB_mirror_get(3,verbose=False)
    return position

def m3r_branch():
    """
    get which branch based on the mirror position and returns the branch
    """
    PV="29id_m3r:"
    TX=caget(PV+"TX_MON")
    if TX > 6:
        branch = 'c'
    elif TX < -1: 
        branch = 'd'
    else:
        print('M3R is not in a default position')
    return branch

def m3r_switch_branch(branch,verbose=True):
    """
    switches the mirror to the position defined in m3r_table
    """
    position = m3r_table(branch)
    if verbose:
        print("Moving M3R to the "+branch+"branch: "+str(position))
    FMB_mirror_move_all(3,position,verbose=False)
    # Relax bellows by doing large Z translation:
    FMB_mirror_move(3,"TY",5,verbose=False)
    sleep(.1)
    FMB_mirror_move(3,"TY",0,verbose=False)
    sleep(.1)
    FMB_mirror_get(3,verbose=True)

##############################################################################################################
################################             M3R alignment             ##############################
##############################################################################################################

def m3r_centroid(t=None,q=1): 
    '''
    Return position of centroid, sigma, m3r:RY (mirror pitch)
    Optional argument t to set camera intergration time.
    '''
    if t is not None:
        caput(M3R_cam1+'AcquireTime',t)
    else:
        t=caget(M3R_cam1+'AcquireTime')
    position =  round(caget(M3R_stats+'CentroidX_RBV'),2)
    sigma = round(caget(M3R_stats+'SigmaX_RBV'),2)
    intensity = round(caget(M3R_stats+'CentroidTotal_RBV'),2)
    m3rRY = round(caget(M3R_pitch),4)
    if q != None:
        print('(position, sigma, total intensity, integration time (s), mirror pitch):')
    return position,sigma,intensity,t,m3rRY

def _m3r_align_start(pxl=None):
    if pxl == None:
        pxl = caget(M3R_align_pv+'desired_pixel')
    caput(M3R_align_pv+'desired_pixel',pxl)
    caput(M3R_align_pv+'startAlign',1,wait=True,timeout=180)

def m3r_align(pxl=None):
    if iex.BL.branch == 'd':
        print('\nalign_m3r()')
        try:
            _m3r_align_start()
            sleep(1)
            if mono_energy_get()>2100:
                _m3r_align_start()
                sleep(1)
            sleep(1)
            min_m3r=caget(M3R_align_pv+'motor_min')
            if m3r_RY_pos_sp_get() == min_m3r:
                align_m3r_epics()
        except:
            print('Unable to align; check camera settings.')