import numpy as np

from epics import caget,caput

from .quick_plot import mda_1D,fit_gauss

from ..instruments.electron_analyzer import scanEA
from ..instruments.slits import slit3C_set
from ..instruments.xrays import energy
from ..instruments.ARPES import mvth,scanx,mvx,mprint


##############################################################################################################
##############################             SES Work Function          ##############################
##############################################################################################################

def WorkFunction_Measure(PE,KE=409,**kwargs):
    '''
    Takes data with 1st and 2nd order light of core level with KE at @ hv=500eV
    **kwargs
    
    '''
    frames=round(17*60*200/PE,0)
    energy(500)
    
    #Fermi Level
    slit3C_set(150)
    kwargs.update({'comment': 'EF1'}) 
    EAlist=['KE',495,PE,frames,1]
    scanEA(EAlist,**kwargs)
    
    #Core level first order
    slit3C_set(50)
    kwargs.update({'comment': 'KE1'})     
    EAlist=['KE',KE,PE,frames/10,1]
    scanEA(EAlist,**kwargs)
    
    #Core level second order
    slit3C_set(150)
    kwargs.update({'comment': 'KE2'}) 
    EAlist=['KE',KE+500,PE,frames*10,1]
    scanEA(EAlist,**kwargs)
    print("""Fit the data :
          - EF1 = EF in first order light (scan #1)
          - KE1 = Au_3/2 in first order light (scan #2)
          - KE2 = Au_3/2 for 2nd order light (scan #3)
          => wk = KE2-2*KE1-(EF1-KE1)`
    Details of the math is page 16 of commissionning book IV
    Now if you know what you are doing you can: 
          caput('29idcEA:det1:ENERGY_OFFSET',new_wk_value)""")
    
def WorkFunction_Calc(EF1,KE1,KE2):
    """Fit the data :
          - EF1 = EF in first order light (scan #1)
          - KE1 = Au_3/2 in first order light (scan #2)
          - KE2 = Au_3/2 for 2nd order light (scan #3)
          => wk = KE2-2*KE1-(EF1-KE1)`
    Details of the math is page 16 of commissionning book IV"""
    wk = KE2-2*KE1-(EF1-KE1)
    print("wk = KE2-2*KE1-(EF1-KE1) = ",wk)
    print("Now if you know what you are doing you can:") 
    print("EA._setwk("+str(round(wk,2))+")")

def WorkFunction(KE1,KE2,Ef):
    Phi=round(KE2-2*KE1-(Ef-KE1),3)
    print(('Phi = '+str(Phi)+' eV'))
    return Phi

def RoughPositions_Find(th_start,th_stop, th_step,**kwargs):
    """automatically scans and creates the RoughPosition list used in FermiMap for 
    relatively uniform samples by:
        1) scanx(-0.5,0.5,.05,mode='relative')
        2) does a gaussian fits the mda EAV intesnity to determine center of the fit x_center
        3) mv(x_center)
        4) mprint() and appends current position to list
        5) moves to the next theta value and repeats
        
    Note: before starting move to the first theta position and adjust the focus leaving the EA is Live Mode
    
    **kwargs
        scanDet=17
    """
    kwargs.setdefault("scanDet",17)
    if caget("29idcEA:det1:Acquire") !=1:
        print("Turn on the EA!!! EALive(PE,KE,Lens=\'Transmission\')")
    else:
        RefinedPositions=[]
        for th in (th_start,th_stop+th_step,th_step):
            mvth(th)
            scanx(-0.5,0.5,.05,mode='relative')
            scanNum=caget("29idARPES:saveData_scanNumber")-1
            x,y,x_name,y_name=mda_1D(scanNum,kwargs["scanDet"])
            results=fit_gauss(np.array(x),np.array(y))
            mvx(results[2][1])  
            RefinedPositions.append(mprint())
        print("RefinedPositions=",RefinedPositions) 
