from math import floor
from epics import caput,PV

import iexcode.instruments.cfg as iex

def scaler_cts(time_seconds=0.1,verbose=True):
    """
    sets the scalers counting for the endstation defined in BL
    """ 
    'For new endstation modify here:'
    if iex.BL.endstation_name == 'kappa':
        Kappa_scaler(time_seconds,verbose=verbose)
    elif iex.BL.endstation_name == 'Octupole':
        Octupole_scaler(time_seconds,verbose)
    else:
        pass

def Kappa_scaler(time_seconds,verbose):
    """
    Sets the integration time for the scalers in the Kappa chamber
    and mpa filter num if mpa is running
    """
    scaler_pv="29idMZ0:scaler1.TP"
    mpa_NumFilter = '29iddMPA:Proc1:NumFilter'
    
    caput(pv,time_seconds)
    
    pv=PV(mpa_NumFilter)
    if pv.connected():
        caput(mpa_NumFilter,max(floor(time_seconds),1))

    if verbose: 
        print("Integration time set to:", str(time_seconds))


def Octupole_scaler(time_seconds,verbose):
    """
    Sets the integration time for the scalers in the Kappa chamber
    and mpa filter num if mpa is running
    """
    scaler_pv="29ide:scaler1.TP" #JM fix here
    
    caput(scaler_pv,time_seconds)

    if verbose: 
        print("Integration time set to:", str(time_seconds))


class Scaler:
    def __init__(self, pv):
        """
        to access scaler pvs
        usage:
            tfy = Scaler('29ide:scaler1.S5')
            tfy.get() => to caget the current value
        """
        self.pv = pv

    def get(self,verbose=True):
        """
        returns the current value
        """
        val = caget(self.pv)
        if verbose:
            print(val)
        return val
