from time import sleep
from epics import caput, caget

import iexcode.instruments.cfg as iex
from iexcode.instruments.scanRecord import saveData_get_all

vortex_pv = "29idVORTEX:"
scaler_pv = '29idMZ0:scaler1.TP'
calc_pv = '29idKappa:userCalcOut3.A'
vortex_power_pv = '29idd:Unidig1Bo1'
transform_pv = '29idd:userTran1'


#Define caput with wait function
def caputw(PVname,PVinput):
    return caput(PVname,PVinput,wait=True,timeout = 100000)


def mca_energy(energy):
    '''
    Sets the energy window for the vortex
    '''
    #energy window calc
    center = 12.8 + 0.72*energy
    width = 120
    #set hi and low
    caput(vortex_pv+'mca1.R0LO',round(center-width/2))
    caput(vortex_pv+'mca1.R0HI',round(center-width/2))
    #pv to store energy set point
    caput(calc_pv,energy)

def mca_scaler(time_sec):
    '''
    Sets acquisition time for vortex and SRS to time_sec
    '''
    caput(vortex_pv+'mca1.PRTM',time_sec)
    caput(scaler_pv,time_sec)
    
def mca_run(time_sec):
    caput(vortex_pv+'mca1.PRTM',time_sec)
    sleep(0.1)
    caput(vortex_pv+'mca1EraseStart',1,wait=True,timeout = 100000)

    
def mca_off():
    '''Power vortex OFF'''
    caput(vortex_power_pv,1)
    print('Vortex power OFF')
    
def mca_on():
    '''Power vortex ON'''
    caput(vortex_power_pv,0)
    print('Vortex power ON')

def _mca_file_setup():
    mda_saveData=saveData_get_all(iex.BL.mda.ioc)
    caput(vortex_pv+'saveData_fileSystem',mda_saveData['fileSystem'])
    caput(vortex_pv+'saveData_subDir', '/'+ mda_saveData['subDir']+'/S'+str(mda_saveData['scanNumber']))
    caput(vortex_pv+'saveData_scanNumber',1)

def _vortex_detector_dictionary(add_mca):
    d = {
        16:vortex_pv+":mca1.R0",
        17:vortex_pv+":mca1.R1",
        49:transform_pv+".D",
        50:transform_pv+".E",
    }
    if add_mca == False:
        for key in d.keys():
            d[key]=''
    return d

def _vortex_trigger_dictionary(save_image,trig_num=2):
    if save_image:
        d = {trig_num:vortex_pv+'scanH.EXSC'}
    else:
        d = {trig_num:vortex_pv+'mca1EraseStart'}
        
    return d

def scanmca(add_mca=True,save_image=False,**kwargs):  
    ''' 
    add_vortex=True (default): set up scan to save vortex as a detector 
    add_vortex=False: remove the vortex from the scan detector
    save_image=True: save the individual mca files
    **kwargs:
        trig_num
    '''
    if trig_num in kwargs:
        trig_num = kwargs['trig_num']
    else:
        trig_num = 2
    if add_mca:
        #add trigger and detectors
        iex.BL.mda.triggers_set(_vortex_trigger_dictionary(save_image,trig_num))
        iex.BL.mda.detectors_set(_vortex_detector_dictionary(add_mca))
        sleep(1)
        if save_image:
            _mca_file_setup()
            mca_check_save()
    else:
        #remove vortex trigger and detectors
        iex.BL.mda.triggers_clear()
        iex.BL.mda.detectors_set(_vortex_detector_dictionary(add_mca))
        sleep(1)

def mca_check_save(**kwargs):
    """
    checks if vortex is being triggered
    **kwargs:
        scan_dim => 1 default
        trig_num => 2 default
    """
    kwargs.setdefault('scan_dim',1)
    kwargs.setdefault('trig_num',2)
    d=iex.BL.mda.triggers_get(kwargs['scan_dim'])
    if d[kwargs['trig_num']] == vortex_pv+'scanH.EXSC':
        print('Saving mca files')
        _mca_file_setup()
        
