from time import sleep

from iexcode.instruments.utilities import *
from iexcode.instruments.kappa import uan
from iexcode.instruments.kappa import tey,d3,d4,mesh,kappa_scaler_pv,tth_pv,tthdet
from iexcode.instruments.Motors import *
from iexcode.instruments.scanRecord import last_mda
from iexcode.macros.quick_plot import *


##### Kappa Optimization scans - 2022 versions #####
 
def Find_kth_zero(th_0,th_180):
    """
    th_0 is the motor position for specular near zero 
    th_180 is motor position for spectular near 180 
    """
    
    Offset =0.5*(180 - th_180 - th_0)

    print("with motor at 0, the actual value is ",Offset)


def align_d4(z0,th):
    ''' th = 0 or 180
    '''
    tthdet.set('d4')
    sleep(1)
    uan(0,th)
    mvz(z0-1500)
    d4.setgain(10,'uA')
    dscantth(-0.25,0.25,0.01)
    sleep(1)
    d4.setgain(1,'nA')
    mvz(z0)
    n=last_mda()
    fit_mda(n,25,0,'poly',graph=False) 
    tth0=fit_d4(n)
    print('tth0 = '+str(round(tth0,3)))
    playsound()
    return tth0


def align_z0_chi0(question=True):
    if question:
        foo=input_d('Are you at th=90/270, chi=0, phi=90 (Y or N)? >')
        if foo == 'Y'.lower() or foo == 'yes'.lower():   
            tthdet.set('d3')
            sleep(1)
            mvtth(0)
            dscanz(-750,750,50)
            sleep(1)
            n=last_mda()
            fit_mda(n,25,0,'poly',graph=False) 
            z0=fit_z(n)
            print('z0 = '+str(round(z0,0)))
            playsound()
            return z0
        else:
            print('Canceling')
            return
    else:
        tthdet.set('d3')
        sleep(1)
        mvtth(0)
        dscanz(-750,750,50)
        sleep(1)
        n=last_mda()
        fit_mda(n,25,0,'poly',graph=False) 
        z0=fit_z(n)
        print('z0 = '+str(round(z0,0)))
        playsound()
        return z0

def align_z0(omega):
    tthdet.set('d3')
    sleep(1)
    uan(0,omega)
    dscanz(-750,750,50)
    sleep(1)
    n=last_mda()
    #fit_mda(n,25,0,'poly',graph=False) 
    #z0=fit_z(n)
    #print('z0 = '+str(round(z0,0)))
    playsound()
    #return z0


def align_z180(omega):
    tthdet.set('d3')
    sleep(1)
    uan(0,180+omega)
    dscanz(-750,750,50)
    sleep(1)
    n=last_mda()
    #fit_mda(n,25,0,'poly',graph=False) 
    #z180=fit_z(n)
    #print('z180 = '+str(round(z180,0)))
    playsound()
    #return z180
    
def align_th0(omega,z):
    tthdet.set('d4')
    sleep(1)
    #uan(20,10+omega)
    uan(30,15+omega)
    mvz(z)
    sleep(30)
    d4.setgain(1,'nA')
    dscanth(-0.25,0.25,0.01)
    n=last_mda()
    fit_mda(n,25,0,'poly',graph=False) 
    th_spec=fit_d4(n)
    print('th specular = '+str(round(th_spec,3)))
    tth=round(caget('29idKappa:m9.RBV'),3)
    omega=th_spec-tth/2
    print('omega = th-tth/2 = '+str(round(omega,3)))
    playsound()
    return omega


def align_th180(omega,z):
    tthdet.set('d4')
    sleep(1)
    uan(-30,180+omega-15)
    #uan(-20,180+omega-10)
    mvz(z)
    d4.setgain(1,'nA')
    dscanth(-0.25,0.25,0.01)
    n=last_mda()
    fit_mda(n,25,0,'poly',graph=False) 
    th_spec=fit_d4(n)
    print('th specular = '+str(round(th_spec,3)))
    tth=round(caget('29idKappa:m9.RBV'),3)
    omega180=th_spec-tth/2-180
    print('omega180 = th-tth/2-180 = '+str(round(omega180,3)))
    playsound()
    return omega180




##### Kappa Optimization scans - old versions #####



def Opt_d4(iteration,moveZ,graph='y',srs=None,start=-0.5,stop=0.5,step=0.04):
    tth_w=0.1
    if srs==None: det=21
    else: det=34
    z=caget('29idKappa:m4.RBV')
    current_det=caget('29idKappa:userStringSeq6.STR1',as_string=True)
    if current_det != 'd4':
        print('Switching detector to d4')
    tthdet.set('d4')
    mvtth(0)
    if moveZ is not None:
        mvz(z-1000)
    for i in range(iteration):
        scantth(start,stop,step,'relative')
        scannum=last_mda()
        tth0=fit_mda(scannum,det,tth_w,'gauss',graph=graph)
        mvtth(tth0)
    mvz(z)
    return tth0,scannum

def Opt_z0(iteration,movetth,movekth,det='d4',srs=None,graph='y'):
    z_w=400
    current_det=caget('29idKappa:userStringSeq6.STR1',as_string=True)
    if current_det != det:
        print('Switching detector to '+det)
    tthdet.set(det)
    if det=='d3' and srs==None: det=20
    if det=='d3' and srs!=None: det=33
    if det=='d4' and srs==None: det=21
    if det=='d4' and srs!=None: det=34
    if movetth is not None:
        mvtth(movetth)  
    if movekth is not None:
        mvkth(movekth)
    if iteration>1:
        scanz(-2000,2000,250,'relative')
        scannum=last_mda()
        z0=fit_mda(scannum,det,z_w,'erf',graph=graph)
        mvz(z0)
    scanz(-700,700,50,'relative')
    scannum=last_mda()
    z0=fit_mda(scannum,det,z_w,'erf',graph=graph)
    mvz(z0)
    return z0,scannum

def Opt_kth(kth_0,theta,det='d4',srs=None,graph='y'):
    current_det=caget('29idKappa:userStringSeq6.STR1',as_string=True)
    if current_det != det:
        print('Switching detector to '+det)
    tthdet.set(det)
    if det == 'd4': 
        if srs==None: det=21
        else: det=34
        i=1
    elif det == 'd3': 
        if srs==None: det=20
        else: det=33
        i=5
    mvtth(theta*2)
    mvkth(kth_0+theta)
    scankth(-0.5*i,0.5*i,0.05*i,'relative')
    new_kth=fit_mda(last_mda(),det,0.1,'gauss',graph=graph)
    mvkth(new_kth)
    scankth(-0.2*i,0.2*i,0.02*i,'relative')
    new_kth=fit_mda(last_mda(),det,0.1,'gauss',graph=graph)
    mvkth(new_kth)
    scannum=last_mda()
    kth0=round(new_kth-theta,3)
    print('\nkth0 = ',kth0)
    print('To plot use:') 
    print('     fit_mda('+str(scannum)+',21,0.2,"gauss",mytitle=\'theta ='+str(theta)+'\')')
    print('If you are happy with that, you can set this value as kth0 using:')
    print('     kth0_set('+str(kth0)+')')

    return kth0,scannum

def plot_opt(opt_scan_list,energy_list,det,mytitle=''):
    fig,(ax1)=plt.subplots(1,1)
    fig.set_size_inches(5,4)

    for i,j in zip(opt_scan_list,energy_list):
        x,y,x_name,y_name=mda_1D(i,det,1,0)
        xdata = np.array(x)
        ydata = np.array(y)
        Imax=np.max(ydata)    
        ax1.plot(x,y/Imax,label=str(j)+" eV")
    ax1.set(ylabel='Intensity')
    ax1.set(xlabel=x_name)
    ax1.ticklabel_format(style='sci', axis='y', scilimits=(0,0))
    ax1.grid(color='lightgray', linestyle='-', linewidth=0.5)
    ax1.legend(bbox_to_anchor=[1.2, 1],ncol=2,shadow=True, title=mytitle, fancybox=True)    
